@@ -3,6 +3,7 @@ from .rdf_to_advanced_graph import rdf_to_advanced_graph
 from .schema_helper import SchemaHelper, SchemaHelperError
 from .transactional import transactional
 from .managed_session import managed_session
+from .constraints import drop_all_fk_constraints
 
 __all__ = [
     "SchemaHelperError",
@@ -10,5 +11,6 @@ __all__ = [
     "compare_json_files",
     "rdf_to_advanced_graph",
     "transactional",
-    "managed_session"
+    "managed_session",
+    "drop_all_fk_constraints"
 ]
